@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dproc.c,v 1.22 2002/01/22 12:40:13 abe Exp $";
+static char *rcsid = "$Id: dproc.c,v 1.23 2003/03/21 17:39:46 abe Exp $";
 #endif
 
 
@@ -103,6 +103,22 @@ so_hash_t **SoHash = (so_hash_t **)NULL;
 
 #define	PROCSIZE	sizeof(struct PROCINFO)
 
+/*
+ * Create the FDSINFOSIZE definition for allocating FDSINFO space.  (This
+ * isn't as straightforward as it might seem, because someone made a bad
+ * decision to change the struct fdsinfo* family at AIX 5.2.)
+ */
+
+#define	FDSINFOSIZE	sizeof(struct FDSINFO)	/* (If we're lucky.) */
+
+#if	defined(OPEN_SHRT_MAX)
+# if	OPEN_SHRT_MAX<OPEN_MAX
+#undef	FDSINFOSIZE				/* (We weren't lucky.) */
+#define	FDSELEMSIZE	(sizeof(struct FDSINFO)/OPEN_SHRT_MAX)
+#define	FDSINFOSIZE	(OPEN_MAX * FDSELEMSIZE)
+# endif	/* OPEN_SHRT_MAX<OPEN_MAX */
+#endif	/* defined(OPEN_SHRT_MAX) */
+
 
 #if	AIXV>=4110
 /*
@@ -210,6 +226,11 @@ ckkv(d, er, ev, ea)
 {
 
 #if	defined(HASKERNIDCK)
+# if	AIXV<5000
+
+/*
+ * Use oslevel below AIX 5.
+ */
 	int br, p[2], pid;
 	char buf[128], *cp;
 	struct stat sb;
@@ -226,10 +247,10 @@ ckkv(d, er, ev, ea)
 	
 	if (stat(OSLEVELPATH, &sb)) {
 
-# if	AIXV<4100
+#  if	AIXV<4100
 	    if (errno == ENOENT)
 		return;
-# endif	/* AIXV<4100 */
+#  endif	/* AIXV<4100 */
 
 	    (void) fprintf(stderr, "%s: can't execute %s: %s\n",
 		Pn, OSLEVELPATH, strerror(errno));
@@ -277,6 +298,19 @@ ckkv(d, er, ev, ea)
 		*cp = '\0';
 	} else
 	    (void) snpf(buf, sizeof(buf), "UNKNOWN");
+# else	/* AIXV>=5000 */
+
+/*
+ * Use uname() for AIX 5 and above.
+ */
+	char buf[64];
+	struct utsname u;
+
+	(void) memset((void *)&u, 0, sizeof(u));
+	(void) uname(&u);
+	(void) snpf(buf, sizeof(buf) - 1, "%s.%s.0.0", u.version, u.release);
+	buf[sizeof(buf) - 1] = '\0';
+# endif	/* AIXV<5000 */
 	if (!ev || strcmp(buf, ev))
 	    (void) fprintf(stderr,
 		"%s: WARNING: compiled for %s version %s; this is %s.\n",
@@ -440,8 +474,7 @@ gather_proc_info()
 	    if (is_cmd_excl(p->pi_comm, &pss, &sf))
 		continue;
 	    if (!fds) {
-		if (!(fds = (struct FDSINFO *)malloc((MALLOC_S)
-					      sizeof(struct FDSINFO))))
+		if (!(fds = (struct FDSINFO *)malloc((MALLOC_S)FDSINFOSIZE)))
 		{
 		    (void) fprintf(stderr,
 			"%s: can't allocate fdsinfo struct for PID %d\n",
@@ -450,8 +483,8 @@ gather_proc_info()
 		}
 	    }
 	    pid = p->p_pid;
-	    if (GETPROCS((struct PROCINFO *)NULL, PROCSIZE, fds,
-			 sizeof(struct FDSINFO), &pid, 1)
+	    if (GETPROCS((struct PROCINFO *)NULL, PROCSIZE, fds, FDSINFOSIZE,
+			  &pid, 1)
 	    != 1)
 		continue;
 	    hl = 0;
@@ -655,13 +688,40 @@ gather_proc_info()
 static void
 get_kernel_access()
 {
+	int oe = 0;
 
 #if	defined(AIX_KERNBITS)
-	int kba;
-	char *kbs;
+	int kb;
+	char *kbb, *kbr;
+/*
+ * Check the kernel bit size against the size for which this executable was
+ * configured.
+ */
+	if (__KERNEL_32()) {
+	    kb = 32;
+	    kbr = "32";
+	} else if (__KERNEL_64()) {
+	    kb = 64;
+	    kbr = "64";
+	} else {
+	    kb = 0;
+	    kbr = "unknown";
+	}
+	if ((AIX_KERNBITS == 0) || !kb || (kb != AIX_KERNBITS)) {
+	    if (AIX_KERNBITS == 32)
+		kbb = "32";
+	    else if (AIX_KERNBITS == 64)
+		kbb = "64";
+	    else
+		kbb = "unknown";
+	    (void) fprintf(stderr,
+		"%s: FATAL: compiled for a kernel of %s bit size.\n", Pn, kbb);
+	    (void) fprintf(stderr,
+		"      The bit size of this kernel is %s.\n", kbr);
+	    Exit(1);
+	}
 #endif	/* defined(AIX_KERNBITS) */
 
-	int oe = 0;
 /*
  * Access /dev/mem.
  */
@@ -756,32 +816,6 @@ get_kernel_access()
  * Check the kernel version number.
  */
 	(void) ckkv("AIX", (char *)NULL, LSOF_VSTR, (char *)NULL);
-
-#if	defined(AIX_KERNBITS)
-/*
- * Check the kernel bit size against the size for which this executable was
- * configured.
- */
-	if (__KERNEL_32()) {
-	    kba = 32;
-	    kbs = "32";
-	} else if (__KERNEL_64()) {
-	    kba = 64;
-	    kbs = "64";
-	} else {
-	    kba = -1;
-	    kbs = "unknown";
-	}
-	if ((kba == -1) || (kba != AIX_KERNBITS)) {
-	    (void) fprintf(stderr,
-		"%s: FATAL: compiled for a %d bit kernel.\n",
-		Pn, AIX_KERNBITS);
-	    (void) fprintf(stderr,
-		"      The bit size of this kernel is %s.\n", kbs);
-	    Exit(1);
-	}
-#endif	/* defined(AIX_KERNBITS) */
-
 }
 
 
