@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.32 2003/06/11 11:44:00 abe Exp $
+ * $Id: dlsof.h,v 1.33 2003/10/07 14:26:41 abe Exp $
  */
 
 
@@ -135,6 +135,10 @@
 
 #include <ufs/ufs/inode.h>
 
+#  if	FREEBSDV>=510
+#undef	i_devvp
+#  endif	/* FREEBSDV>=510 */
+
 #  if	FREEBSDV>=400 && FREEBSDV<500
 #   if	defined(__alpha__) || defined(__sparc64__)
 #undef	dev_t
