@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.29 2003/06/11 11:44:00 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.30 2003/10/07 14:26:41 abe Exp $";
 #endif
 
 
@@ -312,9 +312,9 @@ process_overlaid_node:
 
 #if	defined(HASNCACHE)
 	Lf->na = va;
-# if	defined(HASNCAPID)
+# if	defined(HASNCVPID)
 	Lf->id = v->v_id;
-# endif	/* defined(HASNCAPID) */
+# endif	/* defined(HASNCVPID) */
 #endif	/* defined(HASNCACHE) */
 
 #if	defined(HASFSTRUCT)
