@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.23 2003/03/21 17:41:34 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.24 2004/03/10 23:50:58 abe Exp $";
 #endif
 
 
@@ -136,6 +136,21 @@ process_socket(sa)
 	Lf->lts.rqs = Lf->lts.sqs = 1;
 #endif	/* defined(HASTCPTPIQ) */
 
+#if	defined(HASSOOPT)
+	Lf->lts.ltm = (unsigned int)s.so_linger;
+	Lf->lts.opt = (unsigned int)s.so_options;
+	Lf->lts.qlen = (unsigned int)s.so_qlen;
+	Lf->lts.qlim = (unsigned int)s.so_qlimit;
+	Lf->lts.rbsz = (unsigned long)s.so_rcv.sb_mbmax;
+	Lf->lts.sbsz = (unsigned long)s.so_snd.sb_mbmax;
+	Lf->lts.qlens = Lf->lts.qlims = Lf->lts.rbszs
+		      = Lf->lts.sbszs = (unsigned char)1;
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASSOSTATE)
+	Lf->lts.ss = (unsigned int)s.so_state;
+#endif	/* defined(HASSOSTATE) */
+
 /*
  * Process socket by the associated domain family.
  */
@@ -275,10 +290,16 @@ process_socket(sa)
  	 */
 	    if (fa || la)
 		(void) ent_inaddr(la, lp, fa, fp, fam);
-	    if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
-	    &&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
+	    if (ta && !kread(ta, (char *)&t, sizeof(t))) {
 		Lf->lts.type = 0;
 		Lf->lts.state.i = (int)t.t_state;
+
+#if	defined(HASTCPOPT)
+		Lf->lts.mss = (unsigned long)t.t_maxseg;
+		Lf->lts.msss = (unsigned char)1;
+		Lf->lts.topt = (unsigned int)t.t_flags;
+#endif	/* defined(HASTCPOPT) */
+
 	    }
 	    break;
 /*
