@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.13 2003/03/21 17:42:17 abe Exp $
+ * $Id: machine.h,v 1.14 2003/10/07 14:27:14 abe Exp abe $
  */
 
 
@@ -70,6 +70,14 @@
 #define	CAN_USE_CLNT_CREATE	1
 
 
+/*
+ * DEVDEV_PATH defines the path to the directory that contains device
+ * nodes.
+ */
+
+#define	DEVDEV_PATH	"/dev"
+
+
 /*
  * GET_MAX_FD is defined for those dialects that provide a function other than
  * getdtablesize() to obtain the maximum file descriptor number plus one.
@@ -235,16 +243,12 @@
  * that lsof can search.  A value of 1 directs printname() to prefix the
  * cache value with the file system directory name; 2, avoid the prefix.
  *
- * HASNCAPID is defined for those dialects with a searchable kernel name
- * cache whose cache and vnodes are linked by a capability ID.
- *
  * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
  *
  * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 #define	HASNCACHE	2
-/* #define	HASNCAPID	1	*/
 /* #define	NCACHELDPFX	??? */
 /* #define	NCACHELDSFX	??? */
 
@@ -410,6 +414,17 @@
 /* #define	HASSNODE	1	*/
 
 
+/*
+ * HASSOOPT, HASSOSTATE and HASTCPOPT define the availability of information
+ * on socket options (SO_* symbols), socket states (SS_* symbols) and TCP
+ * options.
+ */
+
+#define	HASSOOPT	1	/* has socket option information */
+#define	HASSOSTATE	1	/* has socket state information */
+/* #define	HASTCPOPT	1	has TCP options or flags */
+
+
 /*
  * Define HASSPECDEVD to be the name of a function that handles the results
  * of a successful stat(2) of a file name argument.
