@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.29 2003/06/11 11:45:13 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.32 2004/01/14 11:29:17 abe Exp $";
 #endif
 
 
@@ -60,16 +60,27 @@ getmemsz(pid)
 	pid_t pid;
 {
 	int n;
-	struct kinfo_proc *p;
 	struct vmspace vm;
 
+#if	defined(HASKVMGETPROC2)
+	struct kinfo_proc2 *p;
+#else	/* !defined(HASKVMGETPROC2) */
+	struct kinfo_proc *p;
+#endif	/* defined(HASKVMGETPROC2) */
+
 	for (n = 0, p = P; n < Np; n++, p++) {
 	    if (p->P_PID == pid) {
 		if (!p->P_VMSPACE
 		||  kread((KA_T)p->P_VMSPACE, (char *)&vm, sizeof(vm)))
 		    return;
+# if	defined(OPENBSDV)
+		Lf->sz = (SZOFFTYPE)((vm.vm_tsize + vm.vm_dsize
+		       + vm.vm_ssize) * sysconf(_SC_PAGESIZE));
+# else	/* !defined(OPENBSDV */
 		Lf->sz = (SZOFFTYPE)ctob(vm.vm_tsize + vm.vm_dsize
 						     + vm.vm_ssize);
+# endif	/* defined(OPENBSDV) */
+
 		Lf->sz_def = 1;
 		return;
 	    }
@@ -228,7 +239,7 @@ process_node(va)
 #if	defined(HASMSDOSFS)
 	struct denode d;
 	u_long dpb;
-	u_long long nn;
+	u_long nn;
 	struct msdosfsmount pm;
 #endif	/* defined(HASMSDOSFS) */
 
@@ -301,9 +312,9 @@ process_overlaid_node:
 
 #if	defined(HASNCACHE)
 	Lf->na = va;
-# if	defined(HASNCAPID)
+# if	defined(HASNCVPID)
 	Lf->id = v->v_id;
-# endif	/* defined(HASNCAPID) */
+# endif	/* defined(HASNCVPID) */
 #endif	/* defined(HASNCACHE) */
 
 #if	defined(HASFSTRUCT)
@@ -499,7 +510,6 @@ process_overlaid_node:
 	    goto process_overlaid_node;
 #endif	/* defined(HASNULLFS) */
 
-
 #if	defined(HASPROCFS)
 	case VT_PROCFS:
 	    if (!v->v_data
@@ -517,6 +527,10 @@ process_overlaid_node:
 	case VT_EXT2FS:
 #endif	/* defined(HASEXT2FS) */
 
+#if	defined(HASLFS)
+	case VT_LFS:
+#endif	/* defined(HASLFS) */
+
 	case VT_UFS:
 	    if (!v->v_data
 	    ||  kread((KA_T)v->v_data, (char *)&i, sizeof(i))) {
@@ -535,7 +549,11 @@ process_overlaid_node:
 		&&  !kread((KA_T)i.i_din.e2fs_din, (char *)&ed, sizeof(ed)))
 		    edp = &ed;
 # else	/* !defined(HASI_E2FS_PTR) */
+#  if	HASEXT2FS<2
 		edp = &i.i_din.e2fs_din;
+#  else	/* HASEXT2FS>=2 */
+		edp = &i.i_e2din;
+#  endif	/* HASEXT2FS>=2 */
 # endif	/* defined(HASI_E2FS_PTR) */
 
 	    } else
@@ -682,7 +700,11 @@ process_overlaid_node:
 		    rdevs = 1;
 		}
 # else	/* !defined(HASI_E2FS_PTR) */
+#  if	HASEXT2FS<2
 		rdev = i.i_din.e2fs_din.e2di_rdev;
+#  else	/* HASEXT2FS>=2 */
+		rdev = i.i_e2din.e2di_rdev;
+#  endif	/* HASEXT2FS>=2 */
 		rdevs = 1;
 # endif	/* defined(HASI_E2FS_PTR) */
 
@@ -729,7 +751,6 @@ process_overlaid_node:
 	    break;
 #endif	/* defined(HASKERNFS) */
 
-
 #if	defined(HAS9660FS)
 	case CDFSNODE:
 	    if (iso_stat) {
@@ -884,13 +905,17 @@ process_overlaid_node:
 
 	    case N_REGLR:
 		if (type == VREG || type == VDIR) {
-		    if (nty == INODE) {
+		    switch (nty) {
+		    case INODE:
 
 #if	defined(HASI_FFS)
+			
 			Lf->sz = (SZOFFTYPE)i.i_ffs_size;
 			Lf->sz_def = 1;
+			break;
 #else	/* !defined(HASI_FFS) */
 # if	defined(HASI_FFS1)
+
 			if (ffs == 1) {
 			    if (u1s) {
 				Lf->sz = (SZOFFTYPE)u1.di_size;
@@ -902,28 +927,30 @@ process_overlaid_node:
 				Lf->sz_def = 1;
 			    }
 			}
+			break;
 # else	/* !defined(HASI_FFS1) */
 			Lf->sz = (SZOFFTYPE)i.i_size;
 			Lf->sz_def = 1;
 # endif	/* defined(HASI_FFS1) */
 #endif	/* defined(HASI_FFS) */
 
-		    }
+			break;
+
 
 #if	defined(HASMSDOSFS)
-		    else if (nty == DOSNODE) {
+		    case DOSNODE:
 			Lf->sz = (SZOFFTYPE)d.de_FileSize;
 			Lf->sz_def = 1;
-		    }
+			break;
 #endif	/* defined(HASMSDOSFS) */
 
-		    else if (nty == MFSNODE) {
+		    case MFSNODE:
 			Lf->sz = (SZOFFTYPE)m.mfs_size;
 			Lf->sz_def = 1;
-		    }
+			break;
 
 #if	defined(HASEXT2FS)
-		    else if (nty == EXT2NODE) {
+		    case EXT2NODE:
 # if	defined(HASI_E2FS_PTR)
 			if (edp) {
 			    Lf->sz = (SZOFFTYPE)edp->e2di_size;
@@ -933,11 +960,11 @@ process_overlaid_node:
 			Lf->sz = (SZOFFTYPE)i.i_e2fs_size;
 			Lf->sz_def = 1;
 # endif	/* defined(HASI_E2FS_PTR) */
-		    }
+			break;
 #endif	/* defined(HASEXT2FS) */
 
-		}
-		else if ((type == VCHR || type == VBLK) && !Fsize)
+		    }
+		} else if ((type == VCHR || type == VBLK) && !Fsize)
 		    Lf->off_def = 1;
 		break;
 	    }
@@ -973,7 +1000,8 @@ process_overlaid_node:
 		}
 		break;
 	    case N_REGLR:
-		if (nty == INODE) {
+		switch (nty) {
+		case INODE:
 
 # if	defined(HASI_FFS)
 		    Lf->nlink = (long)i.i_ffs_nlink;
@@ -998,18 +1026,17 @@ process_overlaid_node:
 # endif	/* defined(HASI_FFS) */
 
 		    Lf->nlink_def = 1;
-		}
+		    break;
 
 #if	defined(HASMSDOSFS)
-		else if (nty == DOSNODE) {
+		case DOSNODE:
 		    Lf->nlink = (long)d.de_refcnt;
 		    Lf->nlink_def = 1;
-		}
+		    break;
 #endif	/* defined(HASMSDOSFS) */
 
-
 #if	defined(HASEXT2FS)
-		else if (nty == EXT2NODE) {
+		case EXT2NODE:
 # if	defined(HASI_E2FS_PTR)
 		    if (edp) {
 			Lf->nlink = (long)edp->e2di_nlink;
@@ -1019,9 +1046,12 @@ process_overlaid_node:
 		    Lf->nlink = (long)i.i_e2fs_nlink;
 		    Lf->nlink_def = 1;
 # endif	/* defined(HASI_E2FS_PTR) */
-		}
-#endif	/* defined(HASEXT2FS) */
 
+		    break;
+	
+#endif	/* defined(HASEXT2FS) */
+		
+		}
 		break;
 	    }
 	    if (Lf->nlink_def && Nlink && (Lf->nlink < Nlink))
