@@ -30,13 +30,39 @@
  */
 
 
+/*
+ * $Id: machine.h,v 1.34 2004/02/24 13:47:19 abe Exp $
+ */
+
+
 #if	!defined(LSOF_MACHINE_H)
 #define	LSOF_MACHINE_H	1
 
 
+# if	solaris>=90000
+/*
+ * <sys/lgrp.h> must be #include'd for Solaris >= 9 before _KMEMUSER or
+ * _KERNEL are defined to avoid a conflict with the use of lgrp_root as
+ * an external symbol in <sys/lgrp.h> and a macro in <sys/lgrp_user.h>.
+ */
+
+#include <sys/lgrp.h>
+# endif	/* solaris>=90000 */
+
+
+# if	solaris>=100000
+/*
+ * Define a dummy aio_req structure for Solaris >= 10, because #include'ing
+ * <sys/aio_req.h> with _KERNEL defined creates too many problems.
+ */
+
+typedef struct aio_req { int dummy; } aio_req_t;
+# endif	/* solaris>=100000 */
+
+
 # if	solaris>=20600
 /*
- * <sys/poll.h> must be #include'd for Solaris 2.6 while _KMEMUSER is
+ * <sys/poll.h> must be #include'd for Solaris >= 2.6 while _KMEMUSER is
  * defined.  Since <netdb.h> also #include's <sys/poll.h> and <netdb.h>
  * is #include'd from lsof.h, we must perform some early #include magic
  * here to set things up properly.
@@ -59,19 +85,26 @@
 # endif	/* solaris>=20600 */
 
 
-/*
- * $Id: machine.h,v 1.31 2002/12/03 18:23:08 abe Exp $
- */
-
-
 /*
  * CAN_USE_CLNT_CREATE is defined for those dialects where RPC clnt_create()
  * can be used to obtain a CLIENT handle in lieu of clnttcp_create().
  */
 
-#if	solaris>=20501
+# if	solaris>=20501
 #define	CAN_USE_CLNT_CREATE	1
-#endif	/* solaris>=20501 */
+# endif	/* solaris>=20501 */
+
+
+/*
+ * DEVDEV_PATH defines the path to the directory that contains device
+ * nodes.
+ */
+
+# if	solaris<100000
+#define	DEVDEV_PATH	"/dev"
+# else	/* solaris>=100000 */
+#define	DEVDEV_PATH	"/devices"
+# endif	/* solaris<100000 */
 
 
 /*
@@ -79,9 +112,9 @@
  * getdtablesize() to obtain the maximum file descriptor number plus one.
  */
 
-#if	solaris<20500
+# if	solaris<20500
 #define	GET_MAX_FD	get_max_fd
-#endif	/* solaris<20500 */
+# endif	/* solaris<20500 */
 
 
 /*
@@ -239,16 +272,12 @@
  * that lsof can search.  A value of 1 directs printname() to prefix the
  * cache value with the file system directory name; 2, avoid the prefix.
  *
- * HASNCAPID is defined for those dialects with a searchable kernel name
- * cache whose cache and vnodes are linked by a capability ID.
- *
  * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
  *
  * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 #define	HASNCACHE	1
-/* #define	HASNCAPID	1	*/
 
 #define	NCACHELDPFX	open_kvm();	/* do before calling ncache_load() */
 
@@ -415,6 +444,19 @@
 #define	HASSNODE	1
 
 
+/*
+ * HASSOOPT, HASSOSTATE and HASTCPOPT define the availability of information
+ * on socket options (SO_* symbols), socket states (SS_* symbols) and TCP
+ * options.
+ */
+
+# if	solaris>=20600
+#define	HASSOOPT	1	/* has socket option information */
+/* #define	HASSOSTATE	1	has socket state information */
+#define	HASTCPOPT	1	/* has TCP options or flags */
+# endif	/* solaris>=20600 */
+
+
 /*
  * Define HASSPECDEVD to be the name of a function that handles the results
  * of a successful stat(2) of a file name argument.
@@ -445,9 +487,9 @@
  * TCP/TPI Recv-Q and Send-Q values produced by netstat.
  */
 
-#if	solaris==20300 || solaris>=20500
+# if	solaris==20300 || solaris>=20500
 #define	HASTCPTPIQ	1
-#endif	/* solaris==20300 || solaris>=20500 */
+# endif	/* solaris==20300 || solaris>=20500 */
 
 
 /*
@@ -455,9 +497,9 @@
  * TCP/TPI send and receive window sizes produced by netstat.
  */
 
-#if	solaris==20300 || solaris>=20500
+# if	solaris==20300 || solaris>=20500
 #define	HASTCPTPIW	1
-#endif	/* solaris==20300 || solaris>=20500 */
+# endif	/* solaris==20300 || solaris>=20500 */
 
 
 /*
