@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.41 2002/04/29 17:13:36 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.45 2004/03/10 23:43:40 abe Exp $";
 #endif
 
 
@@ -82,6 +82,12 @@ main(argc, argv)
 	int sp = 0;
 	struct str_lst *str;
 	int version = 0;
+	int xover = 0;
+
+#if	defined(HASZONES)
+	znhash_t *zp;
+#endif	/* defined(HASZONES) */
+
 /*
  * Save program name.
  */
@@ -131,7 +137,7 @@ main(argc, argv)
  * Create option mask.
  */
 	(void) snpf(options, sizeof(options),
-	    "?a%sbc:D:d:%sf:F:g:hi:%slL:%sMnNo:Op:Pr:%ssS:tT:u:UvVw%s",
+	    "?a%sbc:D:d:%sf:F:g:hi:%slL:%sMnNo:Op:Pr:%ssS:tT:u:UvVwx:%s%s",
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
 	    "A:",
@@ -165,14 +171,20 @@ main(argc, argv)
 
 #if	defined(HASXOPT)
 # if	defined(HASXOPT_ROOT)
-	    (Myuid == 0) ? "X" : ""
+	    (Myuid == 0) ? "X" : "",
 # else	/* !defined(HASXOPT_ROOT) */
-	    "X"
+	    "X",
 # endif	/* defined(HASXOPT_ROOT) */
 #else	/* !defined(HASXOPT) */
-	    ""
+	    "",
 #endif	/* defined(HASXOPT) */
 
+#if	defined(HASZONES)
+	    "z:"
+#else	/* !defined(HASZONES) */
+	    ""
+#endif	/* defined(HASZONES) */
+
 	    );
 /*
  * Loop through options.
@@ -201,11 +213,22 @@ main(argc, argv)
 		Fblock = 1;
 		break;
 	    case 'c':
+		if (GOp == '+') {
+		    if (!GOv || (*GOv == '-') || (*GOv == '+')
+		    ||  !isdigit((int)*GOv))
+		    {
+			(void) fprintf(stderr,
+			    "%s: +c not followed by width number\n", Pn);
+			err = 1;
+		    } else
+			CmdLim = atoi(GOv);
+		    break;
+		}
 		if (GOv && (*GOv == '/')) {
 		    if (enter_cmd_rx(GOv))
 			err = 1;
 		} else {
-		    if (enter_str_lst("c", GOv, &Cmdl))
+		    if (enter_str_lst("-c", GOv, &Cmdl))
 			err = 1;
 		}
 		break;
@@ -220,8 +243,10 @@ main(argc, argv)
 		if (GOp == '+') {
 		    if (enter_dir(GOv, 0))
 			err = 1;
-		    else
+		    else {
 			Selflags |= SELNM;
+			xover = 1;
+		    }
 		} else {
 		    if (enter_fd(GOv))
 			err = 1;
@@ -231,8 +256,10 @@ main(argc, argv)
 		if (GOp == '+') {
 		    if (enter_dir(GOv, 1))
 			err = 1;
-		    else
+		    else {
 			Selflags |= SELNM;
+			xover = 1;
+		    }
 		} else {
 
 #if	defined(HASDCACHE)
@@ -248,6 +275,10 @@ main(argc, argv)
 	    case 'f':
 		if (!GOv || *GOv == '-' || *GOv == '+') {
 		    Ffilesys = (GOp == '+') ? 2 : 1;
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
 		    break;
 		}
 
@@ -321,6 +352,11 @@ main(argc, argv)
 			    continue;
 #endif	/* !defined(HASFSTRUCT) */
 
+#if	!defined(HASZONES)
+			if (FieldSel[i].id == LSOF_FID_ZONE)
+			    continue;
+#endif	/* !defined(HASZONES) */
+
 			if (FieldSel[i].id == LSOF_FID_RDEV)
 			    continue;	/* for compatibility */
 			FieldSel[i].st = 1;
@@ -423,11 +459,11 @@ main(argc, argv)
 	    case 'L':
 		Fnlink = (GOp == '+') ? 1 : 0;
 		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    Nlink = 0l;
 		    if (GOv) {
 			GOx1 = GObk[0];
 			GOx2 = GObk[1];
 		    }
-		    Nlink = 0l;
 		    break;
 		}
 		for (cp = GOv, l = 0l, n = 0; *cp; cp++) {
@@ -510,11 +546,11 @@ main(argc, argv)
 		if (GOp == '+')
 		    ev = rc = 1;
 		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    RptTm = RPTTM;
 		    if (GOv) {
 			GOx1 = GObk[0];
 			GOx2 = GObk[1];
 		    }
-		    RptTm = RPTTM;
 		    break;
 		}
 		for (cp = GOv, i = n = 0; *cp; cp++) {
@@ -544,11 +580,11 @@ main(argc, argv)
 		break;
 	    case 'S':
 		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    TmLimit = TMLIMIT;
 		    if (GOv) {
 			GOx1 = GObk[0];
 			GOx2 = GObk[1];
 		    }
-		    TmLimit = TMLIMIT;
 		    break;
 		}
 		for (cp = GOv, i = n = 0; *cp; cp++) {
@@ -577,16 +613,22 @@ main(argc, argv)
 		break;
 	    case 'T':
 		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    Ftcptpi = (GOp == '-') ? 0 : TCPTPI_STATE;
 		    if (GOv) {
 			GOx1 = GObk[0];
 			GOx2 = GObk[1];
 		    }
-		    Ftcptpi = (GOp == '-') ? 0 : TCPTPI_STATE;
 		    break;
 		}
 		for (Ftcptpi = 0; *GOv; GOv++) {
 		    switch (*GOv) {
 
+#if	defined(HASSOOPT) || defined(HASSOSTATE) || defined(HASTCPOPT)
+		    case 'f':
+			Ftcptpi |= TCPTPI_FLAGS;
+			break;
+#endif	/* defined(HASSOOPT) || defined(HASSOSTATE) || defined(HASTCPOPT) */
+
 #if	defined(HASTCPTPIQ)
 		    case 'q':
 			Ftcptpi |= TCPTPI_QUEUES;
@@ -627,6 +669,32 @@ main(argc, argv)
 	    case 'w':
 		Fwarn = (GOp == '+') ? 0 : 1;
 		break;
+	    case 'x':
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    Fxover = XO_ALL;
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
+		    break;
+		} else {
+		    for (; *GOv; GOv++) {
+			switch (*GOv) {
+			case 'f':
+			    Fxover |= XO_FILESYS;
+			    break;
+			case 'l':
+			    Fxover |= XO_SYMLINK;
+			    break;
+			default:
+			    (void) fprintf(stderr,
+				"%s: unknown cross-over option: %c\n",
+				Pn, *GOv);
+			    err++;
+			}
+		    }
+		}
+		break;
 
 #if	defined(HASXOPT)
 	    case 'X':
@@ -634,6 +702,23 @@ main(argc, argv)
 		break;
 #endif	/* defined(HASXOPT) */
 
+#if	defined(HASZONES)
+	    case 'z':
+		Fzone = 1;
+		if (GOv && (*GOv != '-') && (*GOv != '+')) {
+
+		/*
+		 * Add to the zone name argument hash.
+		 */
+		    if (enter_zone_arg(GOv))
+			err = 1;
+		} else if (GOv) {
+		    GOx1 = GObk[0];
+		    GOx2 = GObk[1];
+		}
+		break;
+#endif	/* defined(HASZONES) */
+
 	    default:
 		(void) fprintf(stderr, "%s: unknown option (%c)\n", Pn, c);
 		err = 1;
@@ -652,6 +737,10 @@ main(argc, argv)
 	    }
 	    FieldSel[LSOF_FIX_PID].st = 1;
 	}
+	if (Fxover && !xover) {
+	    (void) fprintf(stderr, "%s: -x must accompany +d or +D\n", Pn);
+	    err++;
+	}
 	if (DChelp || err || Fhelp || fh || version)
 	    usage(err ? 1 : 0, fh, version);
 /*
@@ -687,6 +776,12 @@ main(argc, argv)
 	    Selflags |= SELUID;
 	if (Nwad)
 	    Selflags |= SELNA;
+
+#if	defined(HASZONES)
+	if (ZoneArg)
+	    Selflags |= SELZONE;
+#endif	/* defined(HASZONES) */
+
 	if (GOx1 < argc)
 	    Selflags |= SELNM;
 	if (Selflags == 0) {
@@ -703,11 +798,11 @@ main(argc, argv)
 	    Selall = 0;
 	}
 /*
- * Get the device for /dev.
+ * Get the device for DEVDEV_PATH.
  */
-	if (stat("/dev", &sb)) {
-	    (void) fprintf(stderr, "%s: can't stat(/dev): %s\n", Pn,
-		strerror(errno));
+	if (stat(DEVDEV_PATH, &sb)) {
+	    (void) fprintf(stderr, "%s: can't stat(%s): %s\n", Pn,
+		DEVDEV_PATH, strerror(errno));
 	    Exit(1);
 	}
 	DevDev = sb.st_dev;
@@ -985,6 +1080,27 @@ main(argc, argv)
 		(void) printf("%s: process ID not located: %d\n",
 		    Pn, Spid[i].i);
 	}
+
+#if	defined(HASZONES)
+	if (ZoneArg) {
+
+	/*
+	 * Check zone argument results.
+	 */
+	    for (i = 0; i < HASHZONE; i++) {
+		for (zp = ZoneArg[i]; zp; zp = zp->next) {
+		    if (!zp->f) {
+			rv = 1;
+			if (Fverbose) {
+			    (void) printf("%s: zone not located: ", Pn);
+			    safestrprt(zp->zn, stdout, 1);
+			}
+		    }
+		}
+	    }
+	}
+#endif	/* defined(HASZONES) */
+
 	for (i = 0; i < Npgid; i++) {
 
 	/*
