@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: proc.c,v 1.35 2002/12/05 12:23:38 abe Exp $";
+static char *rcsid = "$Id: proc.c,v 1.36 2004/03/10 23:44:14 abe Exp $";
 #endif
 
 
@@ -160,6 +160,24 @@ alloc_lfile(nm, num)
 	Lf->lts.type = -1;
 	Lf->nlink = 0l;
 
+#if	defined(HASSOOPT)
+	Lf->lts.kai = Lf->lts.ltm = 0;
+	Lf->lts.opt = Lf->lts.qlen = Lf->lts.qlim = (unsigned int)0;
+	Lf->lts.rbsz = Lf->lts.sbsz = (unsigned long)0;
+	Lf->lts.qlens = Lf->lts.qlims = Lf->lts.rbszs
+		      = Lf->lts.sbszs = (unsigned char)0;
+#endif	/* defined(HASSOOPT) */
+
+#if	defined(HASSOSTATE)
+	Lf->lts.ss = 0;
+#endif	/* defined(HASSOSTATE) */
+
+#if	defined(HASTCPOPT)
+	Lf->lts.mss = (unsigned long)0;
+	Lf->lts.msss = (unsigned char)0;
+	Lf->lts.topt = (unsigned int)0;
+#endif	/* defined(HASTCPOPT) */
+
 #if	defined(HASTCPTPIQ)
 	Lf->lts.rqs = Lf->lts.sqs = (unsigned char)0;
 #endif	/* defined(HASTCPTPIQ) */
@@ -314,6 +332,14 @@ alloc_lproc(pid, pgid, ppid, uid, cmd, pss, sf)
 	    safestrprt(cmd, stderr, 1);
 	    Exit(1);
 	}
+
+#if	defined(HASZONES)
+/*
+ * Clear the zone name pointer.  The dialect's own code will set it.
+ */
+	Lp->zn = (char *)NULL;
+#endif	/* defined(HASZONES) */
+
 }
 
 
@@ -852,6 +878,12 @@ print_proc()
 		return(rv);
 	    rv = 1;
 	    (void) printf("%c%d%c", LSOF_FID_PID, Lp->pid, Terminator);
+
+#if	defined(HASZONES)
+	    if (FieldSel[LSOF_FIX_ZONE].st && Fzone && Lp->zn)
+		(void) printf("%c%s%c", LSOF_FID_ZONE, Lp->zn, Terminator);
+#endif	/* defined(HASZONES) */
+
 	    if (FieldSel[LSOF_FIX_PGID].st && Fpgid)
 		(void) printf("%c%d%c", LSOF_FID_PGID, Lp->pgid, Terminator);
 
